@@ -35,6 +35,7 @@
 
 extern int rss_verbose_debug;
 
+#include "rss-evo-common.h"
 #include "network.h"
 #include "network-soup.h"
 #include "rss.h"
@@ -64,14 +65,25 @@ typedef struct {
 	SoupSession *ss;
 } CallbackInfo;
 
+typedef void (*pCallback)(gpointer data);
+
 typedef struct {
 	SoupSession *ss;
 	SoupMessage *sm;
 	gpointer cb2;
 	gpointer cbdata2;
 	gchar *url;
+	gchar *host;
+	SoupAddress *addr;
+	pCallback callback;
+	gpointer data;
 } STNET;
 
+void idle_callback(gpointer data);
+void queue_callback(gpointer data);
+void proxify_session_async(EProxy *proxy, STNET *stnet);
+static void rss_webkit_resolve_callback (SoupAddress *addr, guint status, gpointer data);
+
 #define DOWNLOAD_QUEUE_SIZE 15
 guint net_qid = 0;		// net queue dispatcher
 guint net_queue_run_count = 0; //downloads in progress
@@ -109,34 +121,18 @@ got_chunk_blocking_cb(SoupMessage *msg, SoupBuffer *chunk, CallbackInfo *info) {
 }
 
 static void
-#if LIBSOUP_VERSION < 2003000
-got_chunk_cb(SoupMessage *msg, CallbackInfo *info) {
-#else
 got_chunk_cb(SoupMessage *msg, SoupBuffer *chunk, CallbackInfo *info) {
-#endif
 
 	NetStatusProgress *progress = NULL;
 	const char* clen;
 
-	if (info->total == 0) {
-#if LIBSOUP_VERSION < 2003000
-		clen = soup_message_get_header(msg->response_headers,
-				"Content-length");
-			return;
-#else
-		clen = soup_message_headers_get(msg->response_headers,
-				"Content-length");
-#endif
-		if (!clen)
-			info->total = 0;
-		else
-			info->total = atoi(clen);
-	}
-#if LIBSOUP_VERSION < 2003000
-	info->current += msg->response.length;
-#else
+	clen = soup_message_headers_get(msg->response_headers,
+			"Content-length");
+	if (!clen)
+		info->total = 0;
+	else
+		info->total = atoi(clen);
 	info->current += chunk->length;
-#endif
 	info->chunk = (gchar *)chunk->data;
 	progress = g_new0(NetStatusProgress, 1);
 
@@ -183,12 +179,10 @@ recv_msg (SoupMessage *msg, gpointer user_data)
 static gboolean
 remove_if_match (gpointer key, gpointer value, gpointer user_data)
 {
-	if (value == user_data)
-	{
+	if (value == user_data) {
 		g_hash_table_remove(rf->key_session, key);
 		return TRUE;
-	}
-	else
+	} else
 		return FALSE;
 }
 
@@ -228,39 +222,62 @@ proxy_init(void)
 	return proxy;
 }
 
+//this will insert proxy in the session
+//and queue loading of page
 void
-proxify_webkit_session(EProxy *proxy, gchar *uri)
+proxify_webkit_session_async(EProxy *proxy, WEBKITNET *wknet)
 {
-	SoupURI *proxy_uri = NULL;
-	gint ptype = gconf_client_get_int (rss_gconf, KEY_GCONF_EVO_PROXY_TYPE, NULL);
+	SoupURI *proxy_uri = NULL, *su;
+	gint ptype = gconf_client_get_int (
+			rss_gconf, KEY_GCONF_EVO_PROXY_TYPE, NULL);
 
 	switch (ptype) {
 #ifndef HAVE_LIBSOUP_GNOME
 	case 0:
 #endif
 	case 2:
-		if (e_proxy_require_proxy_for_uri (proxy, uri)) {
+		su = soup_uri_new (wknet->base);
+		if (su) {
+			if (su->scheme == SOUP_URI_SCHEME_HTTPS) {
+				if (rss_ep_need_proxy_https (proxy, su->host)) {
 #if (DATASERVER_VERSION >=2026000)
-			proxy_uri = e_proxy_peek_uri_for (proxy, uri);
-			d("webkit proxified %s with %s:%d\n", uri, proxy_uri->host, proxy_uri->port);
+					proxy_uri = e_proxy_peek_uri_for (proxy, wknet->base);
 #else
-			g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
-			return;
+					g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
+					soup_uri_free(su);
+					return;
 #endif
-		} else  {
-			d("webkit no PROXY-%s\n", uri);
+					if (proxy_uri) {
+						d("proxified %s with %s:%d\n", wknet->base, proxy_uri->host, proxy_uri->port);
+					}
+				} else {
+					d("no PROXY-%s\n", wknet->base);
+				}
+				g_object_set (
+					G_OBJECT (webkit_session),
+					SOUP_SESSION_PROXY_URI,
+					proxy_uri, NULL);
+				soup_uri_free(su);
+				goto out;
+			} else {
+				wknet->addr = soup_address_new (su->host, 0);
+				soup_uri_free(su);
+				soup_address_resolve_async (wknet->addr, NULL, NULL,
+					rss_webkit_resolve_callback, wknet);
+				return;
+			}
 		}
 		break;
-		g_object_set (
-			G_OBJECT (webkit_session),
-			SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
+
 #ifdef HAVE_LIBSOUP_GNOME
 	case 0:
 		soup_session_add_feature_by_type (
-			webkit_session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+			webkit_session,
+			SOUP_TYPE_PROXY_RESOLVER_GNOME);
 		break;
 #endif
 	}
+out:	wknet->cb(wknet->str, wknet->base, wknet->encoding);
 
 }
 
@@ -298,8 +315,8 @@ proxify_session(EProxy *proxy, SoupSession *session, gchar *uri)
 
 #ifdef HAVE_LIBSOUP_GNOME
 	case 0:
-			soup_session_add_feature_by_type (
-				session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+		soup_session_add_feature_by_type (
+			session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
 		break;
 #endif
 	}
@@ -307,6 +324,128 @@ proxify_session(EProxy *proxy, SoupSession *session, gchar *uri)
 }
 #endif
 
+static void
+rss_resolve_callback (SoupAddress *addr, guint status, gpointer data)
+{
+	STNET *stnet = (STNET *)data;
+	SoupURI *proxy_uri = NULL;
+	if (status == SOUP_STATUS_OK) {
+		if (rss_ep_need_proxy_http (proxy, stnet->host, stnet->addr)) {
+#if (DATASERVER_VERSION >=2026000)
+			proxy_uri = e_proxy_peek_uri_for (proxy, stnet->url);
+#else
+			g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
+			return;
+#endif
+			if (proxy_uri) {
+				d("proxified %s with %s:%d\n", stnet->url, proxy_uri->host, proxy_uri->port);
+			}
+		}
+	} else {
+		d("no PROXY-%s\n", stnet->url);
+	}
+	g_object_set (
+		G_OBJECT (stnet->ss),
+		SOUP_SESSION_PROXY_URI,
+		proxy_uri, NULL);
+	stnet->callback(stnet->data);
+}
+
+static void
+rss_webkit_resolve_callback (SoupAddress *addr, guint status, gpointer data)
+{
+	WEBKITNET *wknet = (WEBKITNET *)data;
+	SoupURI *proxy_uri = NULL;
+	if (status == SOUP_STATUS_OK) {
+		if (rss_ep_need_proxy_http (proxy, wknet->host, wknet->addr)) {
+#if (DATASERVER_VERSION >=2026000)
+			proxy_uri = e_proxy_peek_uri_for (proxy, wknet->base);
+#else
+			g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
+			return;
+#endif
+			if (proxy_uri) {
+				dp("proxified %s with %s:%d\n", wknet->base, proxy_uri->host, proxy_uri->port);
+			}
+		}
+	} else {
+		d("no PROXY-%s\n", wknet->base);
+	}
+	g_object_set (
+		G_OBJECT (webkit_session),
+		SOUP_SESSION_PROXY_URI,
+		proxy_uri, NULL);
+	wknet->cb(wknet->str, wknet->base, wknet->encoding);
+}
+
+void
+queue_callback(gpointer data)
+{
+	STNET *stnet = (STNET *)data;
+	soup_session_queue_message (stnet->ss, stnet->sm,
+		stnet->cb2, stnet->cbdata2);
+}
+
+//this will insert proxy in the session
+//and queue the message to be sent
+void
+proxify_session_async(EProxy *proxy, STNET *stnet)
+{
+	SoupURI *proxy_uri = NULL, *su;
+	gint ptype = gconf_client_get_int (
+			rss_gconf, KEY_GCONF_EVO_PROXY_TYPE, NULL);
+
+	switch (ptype) {
+#ifndef HAVE_LIBSOUP_GNOME
+	case 0:
+#endif
+	case 2:
+		su = soup_uri_new (stnet->url);
+		stnet->host = su->host;
+		if (su) {
+			if (su->scheme == SOUP_URI_SCHEME_HTTPS) {
+				if (rss_ep_need_proxy_https (proxy, su->host)) {
+#if (DATASERVER_VERSION >=2026000)
+					proxy_uri = e_proxy_peek_uri_for (proxy, stnet->url);
+#else
+					g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
+					soup_uri_free(su);
+					return;
+#endif
+					if (proxy_uri) {
+						d("proxified %s with %s:%d\n", stnet->url, proxy_uri->host, proxy_uri->port);
+					}
+				} else {
+					d("no PROXY-%s\n", stnet->url);
+				}
+				g_object_set (
+					G_OBJECT (stnet->ss),
+					SOUP_SESSION_PROXY_URI,
+					proxy_uri, NULL);
+				soup_uri_free(su);
+				goto out;
+			} else {
+				stnet->addr = soup_address_new (su->host, 0);
+				soup_uri_free(su);
+				soup_address_resolve_async (stnet->addr, NULL, NULL,
+					rss_resolve_callback, stnet);
+				return;
+			}
+		}
+		break;
+
+#ifdef HAVE_LIBSOUP_GNOME
+	case 0:
+		soup_session_add_feature_by_type (
+			stnet->ss, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+		break;
+#endif
+	}
+
+out:	stnet->callback(stnet->data);
+	//free stnet
+}
+
 guint
 read_up(gpointer data)
 {
@@ -416,7 +555,7 @@ authenticate (SoupSession *session,
 	RSS_AUTH *auth_info = g_new0(RSS_AUTH, 1);
 
 	if (msg->status_code == SOUP_STATUS_PROXY_UNAUTHORIZED) {
-		g_print("proxy:%d\n", soup_auth_is_for_proxy(auth));
+		d("proxy:%d\n", soup_auth_is_for_proxy(auth));
 	g_object_get (G_OBJECT(session),
 				"proxy-uri", &proxy_uri,
 				NULL);
@@ -609,7 +748,6 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 	}
 }
 
-
 gboolean
 net_get_unblocking(gchar *url,
 			NetStatusCallback cb, gpointer data,
@@ -621,6 +759,9 @@ net_get_unblocking(gchar *url,
 	CallbackInfo *info = NULL;
 	SoupSession *soup_sess;
 	gchar *agstr;
+	gchar *mainurl = NULL;
+	gchar **res;
+	STNET *stnet;
 
 	soup_sess = soup_session_async_new();
 
@@ -632,9 +773,6 @@ net_get_unblocking(gchar *url,
 	}
 #endif
 
-#if (DATASERVER_VERSION >= 2023001)
-	proxify_session(proxy, soup_sess, url);
-#endif
 	if (cb && data) {
 		info = g_new0(CallbackInfo, 1);
 		info->user_cb = cb;
@@ -643,17 +781,26 @@ net_get_unblocking(gchar *url,
 		info->total = 0;
 		info->ss = soup_sess;
 	}
+	/*try to find upstream url to supply a password*/
+	if (data) {
+		res = g_strsplit(data, ";COMMENT-", 0);
+		if (res[0] && g_str_has_prefix(res[0], "RSS-")) {
+			mainurl = g_strdup(res[0]+4);
+			g_strfreev(res);
+		}
+	}
 
 	g_signal_connect (soup_sess, "authenticate",
-		G_CALLBACK (authenticate), (gpointer)url);
+		G_CALLBACK (authenticate), (gpointer)mainurl?mainurl:url);
 #if LIBSOUP_VERSION < 2003000
 	g_signal_connect (soup_sess, "reauthenticate",
-		G_CALLBACK (reauthenticate), (gpointer)url);
+		G_CALLBACK (reauthenticate), (gpointer)mainurl?mainurl:url);
 #endif
 
 	/* Queue an async HTTP request */
 	msg = soup_message_new ("GET", url);
 	if (!msg) {
+		if (info) g_free(info);
 		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(2));			//invalid url
 		return FALSE;
@@ -686,18 +833,36 @@ net_get_unblocking(gchar *url,
 			"Location", G_CALLBACK (redirect_handler), info);
 	}
 
+	stnet = g_new0(STNET, 1);
+	stnet->ss = soup_sess;
+	stnet->sm = msg;
+	stnet->cb2 = cb2;
+	stnet->cbdata2 = cbdata2;
+	stnet->url = g_strdup(url);
+	stnet->callback = queue_callback;
+	stnet->data = stnet;
 
-	soup_session_queue_message (soup_sess, msg,
-		cb2, cbdata2);
+	proxify_session_async(proxy, stnet);
+	//free stnet
 
 ////	g_object_add_weak_pointer (G_OBJECT(msg), (gpointer)info);
 	g_object_weak_ref (G_OBJECT(msg), unblock_free, soup_sess);
 //	g_object_weak_ref (G_OBJECT(soup_sess), unblock_free, soup_sess);
-//	GMainLoop *mainloop = g_main_loop_new (g_main_context_default (), FALSE);
-//	g_timeout_add (10 * 1000, &conn_mainloop_quit, mainloop);
+	if (mainurl)
+		g_free(mainurl);
 	return TRUE;
 }
 
+void
+idle_callback(gpointer data)
+{
+	STNET *stnet = (STNET *)data;
+	g_queue_push_tail (rf->stqueue, stnet);
+	rf->enclist = g_list_append (rf->enclist, stnet->url);
+
+	if (!net_qid)
+		net_qid = g_idle_add((GSourceFunc)net_queue_dispatcher, NULL);
+}
 
 // same stuff as net_get_* but without accumulating headers
 // push all donwloads to a customizable length queue
@@ -727,9 +892,6 @@ download_unblocking(
 	}
 #endif
 
-#if (DATASERVER_VERSION >= 2023001)
-	proxify_session(proxy, soup_sess, url);
-#endif
 	if (cb && data) {
 		info = g_new0(CallbackInfo, 1);
 		info->user_cb = cb;
@@ -749,6 +911,7 @@ download_unblocking(
 	/* Queue an async HTTP request */
 	msg = soup_message_new ("GET", url);
 	if (!msg) {
+		g_free(info);
 		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(2));			//invalid url
 		return FALSE;
@@ -776,24 +939,26 @@ download_unblocking(
 	if (info) {
 		g_signal_connect(G_OBJECT(msg), "got_chunk",
 			G_CALLBACK(got_chunk_cb), info);	//FIXME Find a way to free this maybe weak_ref
+		soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
+		soup_message_add_header_handler (msg, "got_body",
+			"Location", G_CALLBACK (redirect_handler), info);
 	}
 
-	soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
-	soup_message_add_header_handler (msg, "got_body",
-		"Location", G_CALLBACK (redirect_handler), info);
-
 	soup_message_body_set_accumulate (msg->response_body, FALSE);
 	stnet = g_new0(STNET, 1);
 	stnet->ss = soup_sess;
 	stnet->sm = msg;
 	stnet->cb2 = cb2;
 	stnet->cbdata2 = cbdata2;
-	stnet->url = g_strdup(url);
-	g_queue_push_tail (rf->stqueue, stnet);
-	rf->enclist = g_list_append (rf->enclist, g_strdup(url));
+	stnet->url = url;
+	stnet->callback = idle_callback;
+	stnet->data = stnet;
+
 	if (!net_qid)
 		net_qid = g_idle_add((GSourceFunc)net_queue_dispatcher, NULL);
 
+	stnet->callback(stnet->data);
+
 ////	g_object_add_weak_pointer (G_OBJECT(msg), (gpointer)info);
 	g_object_weak_ref (G_OBJECT(msg), unblock_free, soup_sess);
 //	g_object_weak_ref (G_OBJECT(soup_sess), unblock_free, soup_sess);
@@ -945,9 +1110,9 @@ out:
 }
 
 gboolean
-cancel_soup_sess(gpointer key, gpointer value, gpointer user_data)
+abort_soup_sess(gpointer key, gpointer value, gpointer user_data)
 {
-	if (SOUP_IS_SESSION(key)) {
+	if (key && SOUP_IS_SESSION(key)) {
 		soup_session_abort(key);
 		g_hash_table_find(rf->key_session,
 			remove_if_match,
@@ -972,9 +1137,9 @@ abort_all_soup(void)
 	rf->cancel_all = 1;
 	if (rf->abort_session) {
 		g_hash_table_foreach(rf->abort_session, remove_weak, NULL);
-		g_hash_table_foreach_remove(
-			rf->abort_session, cancel_soup_sess, NULL);
-//              g_hash_table_foreach(rf->abort_session, cancel_soup_sess, NULL);
+		if (g_hash_table_size(rf->abort_session))
+			g_hash_table_foreach_remove(
+				rf->abort_session, abort_soup_sess, NULL);
 		g_hash_table_destroy(rf->session);
 		rf->session = g_hash_table_new(
 				g_direct_hash, g_direct_equal);
