@@ -29,11 +29,6 @@
 #include <e-util/e-error.h>
 #endif
 
-#if EVOLUTION_VERSION <= 22203
-#include <misc/e-activity-handler.h>
-#include <e-util/e-icon-factory.h>
-#endif
-
 extern int rss_verbose_debug;
 
 #include "rss.h"
@@ -45,12 +40,14 @@ extern EShellView *rss_shell_view;
 #endif
 extern rssfeed *rf;
 
+#if (EVOLUTION_VERSION < 29102)
 static void
 dialog_key_destroy (GtkWidget *widget, gpointer data)
 {
 	if (data)
 		g_hash_table_remove(rf->error_hash, data);
 }
+#endif
 
 void
 err_destroy (GtkWidget *widget, guint response, gpointer data)
@@ -62,11 +59,14 @@ err_destroy (GtkWidget *widget, guint response, gpointer data)
 void
 rss_error(gpointer key, gchar *name, gchar *error, gchar *emsg)
 {
-	GtkWidget *ed;
+	GtkWidget *ed = NULL;
 	gchar *msg;
+#if (EVOLUTION_VERSION < 29102)
 	gpointer newkey;
+#endif
 #if (EVOLUTION_VERSION >= 22900) //kb//
 	EShell *shell;
+	EMailBackend *backend;
 	GtkWindow *parent;
 	GList *windows;
 #else
@@ -79,22 +79,28 @@ rss_error(gpointer key, gchar *name, gchar *error, gchar *emsg)
 	else
 		msg = g_strdup(emsg);
 
-#if (EVOLUTION_VERSION >= 22200)
 	if (key) {
 		if (!g_hash_table_lookup(rf->error_hash, key)) {
-//			guint activity_id = g_hash_table_lookup(rf->activity, key);
 #if (EVOLUTION_VERSION >= 22900) //kb//
 			shell = e_shell_get_default ();
 			windows = e_shell_get_watched_windows (shell);
 			parent = (windows != NULL) ? GTK_WINDOW (windows->data) : NULL;
 
+#if (EVOLUTION_VERSION >= 29102)
+			backend = (EMailBackend *)e_shell_get_backend_by_name (shell, "mail");
+			e_mail_backend_submit_alert (
+				backend, "org-gnome-evolution-rss:feederr",
+				error, msg, NULL);
+#else
 			ed = e_alert_dialog_new_for_args(parent,
 				"org-gnome-evolution-rss:feederr",
 				error, msg, NULL);
+#endif
 #else
 			ed = e_error_new(NULL, "org-gnome-evolution-rss:feederr",
 				error, msg, NULL);
 #endif
+#if (EVOLUTION_VERSION < 29102)
 			newkey = g_strdup(key);
 			g_signal_connect(
 				ed, "response",
@@ -108,10 +114,12 @@ rss_error(gpointer key, gchar *name, gchar *error, gchar *emsg)
 				G_CALLBACK(dialog_key_destroy),
 				newkey);
 			//lame widget destruction, seems e_activity timeout does not destroy it
-			g_timeout_add_seconds(60, 
+			g_timeout_add_seconds(60,
 				(GSourceFunc)gtk_widget_destroy,
 				ed);
+#endif
 
+#if (EVOLUTION_VERSION < 29102)
 #if (EVOLUTION_VERSION >= 22900) //kb//
 		em_utils_show_error_silent(ed);
 		g_hash_table_insert(
@@ -120,29 +128,21 @@ rss_error(gpointer key, gchar *name, gchar *error, gchar *emsg)
 			GINT_TO_POINTER(1));
 
 #else
-		activity_handler = 
+		activity_handler =
 			mail_component_peek_activity_handler (mail_component_peek());
-#if (EVOLUTION_VERSION >= 22203)
 		id = e_activity_handler_make_error (
 			activity_handler,
 			(char *)mail_component_peek(),
 			E_LOG_ERROR,
 			ed);
-#else
-		id = e_activity_handler_make_error (
-			activity_handler,
-			(char *)mail_component_peek(),
-			(gchar *)msg,
-			ed);
-#endif
 		g_hash_table_insert(rf->error_hash,
 			newkey,
 			GINT_TO_POINTER(id));
+#endif
 #endif
 		}
 		goto out;
 	}
-#endif
 
 	if (!rf->errdialog) {
 #if (EVOLUTION_VERSION >= 22900) //kb//
@@ -180,8 +180,7 @@ taskbar_push_message(gchar *message)
 	e_activity_handler_set_message(activity_handler, message);
 #else
 	EShellTaskbar *shell_taskbar;
-	if (!rss_shell_view)
-		return;
+	g_return_if_fail(rss_shell_view != NULL);
 	shell_taskbar = e_shell_view_get_shell_taskbar (rss_shell_view);
 	e_shell_taskbar_set_message (shell_taskbar, message);
 #endif
@@ -203,7 +202,11 @@ taskbar_pop_message(void)
 }
 
 void
+#if EVOLUTION_VERSION > 29102
+taskbar_op_abort(CamelOperation *cancellable, gpointer key)
+#else
 taskbar_op_abort(gpointer key)
+#endif
 {
 #if EVOLUTION_VERSION < 22900 //kb//
 	EActivityHandler *activity_handler =
@@ -214,9 +217,13 @@ taskbar_op_abort(gpointer key)
 		e_activity_handler_operation_finished(
 				activity_handler,
 				activity_key);
-#endif
+#else
+	EActivity *activity = g_hash_table_lookup(rf->activity, key);
+	e_activity_set_state (activity, E_ACTIVITY_CANCELLED);
 	g_hash_table_remove(rf->activity, key);
+	g_object_unref(activity);
 	abort_all_soup();
+#endif
 }
 
 #if EVOLUTION_VERSION >= 22900 //kb//
@@ -224,56 +231,57 @@ EActivity *
 #else
 guint
 #endif
-#if (EVOLUTION_VERSION >= 22200)
 taskbar_op_new(gchar *message, gpointer key);
-#else
-taskbar_op_new(gchar *message);
-#endif
 
 #if EVOLUTION_VERSION >= 22900 //kb//
 EActivity *
 #else
 guint
 #endif
-#if (EVOLUTION_VERSION >= 22200)
 taskbar_op_new(gchar *message, gpointer key)
-#else
-taskbar_op_new(gchar *message)
-#endif
 {
 #if EVOLUTION_VERSION >= 22900 //kb//
 	EShell *shell;
 	EShellBackend *shell_backend;
 	EActivity *activity;
+	GCancellable *cancellable;
 #else
 	EActivityHandler *activity_handler;
 	char *mcp;
 	guint activity_id;
 #endif
-#if EVOLUTION_VERSION < 22306
-	GdkPixbuf *progress_icon;
-#endif
 
 #if EVOLUTION_VERSION >= 22900 //kb//
 
 	shell = e_shell_get_default ();
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
 
+#if EVOLUTION_VERSION >= 23300
+	activity = e_activity_new ();
+#if (EVOLUTION_VERSION >= 29102)
+	e_activity_set_text (activity, message);
+#else
+	e_activity_set_primary_text (activity, message);
+#endif
+	cancellable = camel_operation_new ();
+	e_activity_set_cancellable (activity, cancellable);
+#else
 	activity = e_activity_new (message);
 	e_activity_set_allow_cancel (activity, TRUE);
+#endif
 	e_activity_set_percent (activity, 0.0);
 	e_shell_backend_add_activity (shell_backend, activity);
 
 	g_signal_connect (
-		activity, "cancelled",
+		cancellable, "cancelled",
 		G_CALLBACK (taskbar_op_abort),
 		key);
+	g_object_unref (cancellable);
 	return activity;
 #else
 	activity_handler =
 		mail_component_peek_activity_handler (mail_component_peek ());
 	mcp = g_strdup_printf("%p", mail_component_peek());
-#if (EVOLUTION_VERSION >= 22306)
 	activity_id =
 		e_activity_handler_cancelable_operation_started(
 			activity_handler,
@@ -282,29 +290,6 @@ taskbar_op_new(gchar *message)
 			TRUE,
 			(void (*) (gpointer))taskbar_op_abort,
 			key);
-#else
-	progress_icon =
-		e_icon_factory_get_icon (
-			"mail-unread",
-			E_ICON_SIZE_MENU);
-#if (EVOLUTION_VERSION >= 22200)
-	activity_id =
-		e_activity_handler_cancelable_operation_started(
-			activity_handler,
-			"evolution-mail",
-			progress_icon,
-			message, TRUE,
-			(void (*) (gpointer))taskbar_op_abort,
-			key);
-#else
-	e_activity_handler_operation_started(
-		activity_handler,
-		mcp,
-		progress_icon,
-		message,
-		FALSE);
-#endif
-#endif
 	g_free(mcp);
 	return activity_id;
 #endif //kb//
@@ -372,7 +357,12 @@ taskbar_op_finish(gchar *key)
 		if (activity_key) {
 			d("activity_key:%p\n", (gpointer)activity_key);
 #if (EVOLUTION_VERSION >= 22900) //kb//
+#if (EVOLUTION_VERSION >= 29102)
+			e_activity_set_state (activity_key, E_ACTIVITY_COMPLETED);
+			g_object_unref(activity_key);
+#else
 			e_activity_complete (activity_key);
+#endif
 #else
 			e_activity_handler_operation_finished(activity_handler, activity_key);
 #endif
@@ -380,7 +370,12 @@ taskbar_op_finish(gchar *key)
 		}
 	} else {
 #if (EVOLUTION_VERSION >= 22900) //kb//
+#if (EVOLUTION_VERSION >= 29102)
+		e_activity_set_state (aid, E_ACTIVITY_COMPLETED);
+		g_object_unref(aid);
+#else
 		e_activity_complete (aid);
+#endif
 #else
 		e_activity_handler_operation_finished(activity_handler, aid);
 #endif
@@ -399,15 +394,13 @@ taskbar_op_message(gchar *msg, gchar *unikey)
 #if (EVOLUTION_VERSION >= 22900) //kb//
 		EActivity *activity_id;
 #else
-#if (EVOLUTION_VERSION >= 22200)
-		guint activity_id;
 #endif
-#endif
-		if (!msg)
+		if (!msg) {
 			tmsg = g_strdup_printf(
 				_("Fetching Feeds (%d enabled)"),
 				g_hash_table_size(rf->hrname));
-		else
+			unikey = (gchar *)"main";
+		} else
 			tmsg = g_strdup(msg);
 
 #if (EVOLUTION_VERSION >= 22900) //kb//
@@ -415,31 +408,20 @@ taskbar_op_message(gchar *msg, gchar *unikey)
 			activity_id =
 				(EActivity *)taskbar_op_new(
 					tmsg,
-					(gchar *)"main");
+					unikey);
 		else
 			activity_id =
 				(EActivity *)taskbar_op_new(tmsg, msg);
 #else
-#if (EVOLUTION_VERSION >= 22200)
 		if (!msg)
-			activity_id = taskbar_op_new(tmsg, (gchar *)"main");
+			activity_id = taskbar_op_new(tmsg, unikey);
 		else
 			activity_id = taskbar_op_new(tmsg, msg);
-#else
-		activity_id = taskbar_op_new(tmsg);
 #endif
-#endif
-		if (!msg)
-			g_hash_table_insert(
-				rf->activity,
-				(gchar *)"main",
-				GUINT_TO_POINTER(activity_id));
-		else
-			if (unikey)
-				g_hash_table_insert(
-					rf->activity,
-					GUINT_TO_POINTER(unikey),
-					GUINT_TO_POINTER(activity_id));
+		g_hash_table_insert(
+			rf->activity,
+			GUINT_TO_POINTER(unikey),
+			GUINT_TO_POINTER(activity_id));
 		g_free(tmsg);
 		return activity_id;
 }
