@@ -31,6 +31,7 @@
 #else
 #include <camel/camel-url.h>
 #endif
+#include <libedataserver/e-data-server-util.h>
 
 extern int rss_verbose_debug;
 
@@ -38,10 +39,20 @@ extern int rss_verbose_debug;
 #include "rss.h"
 #include "rss-config.h"
 #include "rss-image.h"
+#include "rss-status-icon.h"
 #include "parser.h"
 #include "misc.h"
 #include "network-soup.h"
 
+typedef struct {
+	RDF *r;
+	GQueue *status_msg;
+} AsyncData;
+
+extern GConfClient *rss_gconf;
+void asyncr_context_free(AsyncData *asyncr);
+GQueue *display_channel_items_sync(AsyncData *ayncr);
+
 /************ RDF Parser *******************/
 
 guint rsserror = FALSE;
@@ -69,14 +80,15 @@ rss_html_url_decode(const char *html, int len)
 	while ((doc = (xmlDoc *)html_find((xmlNode *)doc, (gchar *)"img"))) {
 		if ((url = (gchar *)xmlGetProp((xmlNodePtr)doc, (xmlChar *)"src"))) {
 			if (strstr(url, "img:")) {
-				gchar *tmp = decode_image_cache_filename(url);
-				duri = g_strconcat("file://", tmp, NULL);
-				g_free(tmp);
+					gchar *tmp = decode_image_cache_filename(url);
+					duri = g_strconcat("file://", tmp, NULL);
+					g_free(tmp);
 			}
 			xmlSetProp(
 				(xmlNodePtr)doc,
 				(xmlChar *)"src",
 				(xmlChar *)duri);
+			xmlFree(url);
 		}
 	}
 	return src;
@@ -539,7 +551,6 @@ layer_find_tag (xmlNodePtr node,
 {
 	xmlBufferPtr buf = xmlBufferCreate();
 	gchar *content;
-	guint len = 0;
 	int i;
 	char* (*func)();
 
@@ -566,7 +577,7 @@ layer_find_tag (xmlNodePtr node,
 					//we need separate xhtml parsing because of xmlNodegetcontent substitutes html entities
 					if (nodetype && !strcasecmp(nodetype, "xhtml")) {		// test this with "html" or smth else
 						//this looses html entities
-						len = xmlNodeDump(buf, node->doc, node, 0, 0);
+						xmlNodeDump(buf, node->doc, node, 0, 0);
 						content = g_strdup_printf("%s", xmlBufferContent(buf));
 					} else {
 						content = (char *)xmlNodeGetContent(node);
@@ -702,13 +713,45 @@ layer_query_find_prop (xmlNodePtr node,
 				}
 			}
 			xmlFree(tprop);
-		return (gchar *)xmlGetProp(node, prop);
+			return (gchar *)xmlGetProp(node, prop);
 		}
 next:		node = node->next;
 	}
 	return NULL;
 }
 
+/*
+ * like layer_query_find_prop but locates all properties in an entry
+ */
+
+GList *
+layer_query_find_all_prop (xmlNodePtr node,
+		const char *match,
+		xmlChar *attr,
+		const char *attrprop,
+		xmlChar *prop)
+{
+	gchar *tmp;
+	GList *result = NULL;
+	while (node!=NULL) {
+		if (!g_ascii_strcasecmp((gchar *)node->name, match)) {
+			gchar *tprop = (gchar *)xmlGetProp(node, attr);
+			if (tprop) {
+				if (g_ascii_strcasecmp(tprop, attrprop)) {
+					xmlFree(tprop);
+					goto next;
+				}
+			}
+			xmlFree(tprop);
+			tmp =  (gchar *)xmlGetProp(node, prop);
+			if (tmp)
+				result = g_list_append(result, tmp);
+		}
+next:		node = node->next;
+	}
+	return result;
+}
+
 char *
 layer_find_url (xmlNodePtr node,
 		char *match,
@@ -768,21 +811,8 @@ tree_walk (xmlNodePtr root, RDF *r)
 	xmlNodePtr channel = NULL, image = NULL;
 	GArray *item = g_array_new (TRUE, TRUE, sizeof (xmlNodePtr));
 	gchar *t;
-	gchar *charset;
 	gchar *md2, *tmp, *ver;
 
-	/* check in-memory encoding first,
-	 * fallback to transport encoding, which may or may not be correct
-	 */
-	if (r->cache->charset == XML_CHAR_ENCODING_UTF8
-	    || r->cache->charset == XML_CHAR_ENCODING_ASCII) {
-		charset = NULL;
-	} else {
-		/* bad/missing encoding, fallback to latin1 (locale?) */
-		charset = r->cache->encoding ?
-			(gchar *)r->cache->encoding : (gchar *)"iso-8859-1";
-	}
-
 	do {
 		walk = rewalk;
 		rewalk = NULL;
@@ -970,30 +1000,6 @@ process_images(gchar *text, gchar *link, gboolean decode, EMFormatHTML *format)
 	return g_strdup(text);
 }
 
-static void
-saxCharacters (void *user_data, const xmlChar *string, int len)
-{
-	GString *buffer = (GString *)user_data;
-	g_string_append_len(buffer, (gchar *)string, len);
-}
-
-GString*
-rss_strip_html (gchar *string)
-{
-	htmlParserCtxtPtr       ctxt;
-	htmlSAXHandlerPtr       sax;
-	GString *buffer = g_string_new(NULL);
-
-	sax = g_new0 (htmlSAXHandler, 1);
-	sax->characters = saxCharacters;
-	ctxt = htmlCreatePushParserCtxt
-		(sax, buffer, string, strlen (string), "", XML_CHAR_ENCODING_UTF8);
-	htmlParseChunk (ctxt, string, 0, 1);
-	htmlFreeParserCtxt (ctxt);
-	g_free (sax);
-	return buffer;
-}
-
 create_feed *
 parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 {
@@ -1020,12 +1026,11 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 	//we have to free this somehow
 	//<link></link>
 	link = g_strdup(layer_find (top, "link", NULL));		//RSS,
-	if (!link) {								// <link href=>
+	if (!link) {                                                            // <link href=>
 		if (!(link = layer_query_find_prop (top,
-				"link",
-				(xmlChar *)"rel",
-				"alternate", (xmlChar *)"href")))	//ATOM
-			link = g_strdup(_("No Information"));
+			"link",(xmlChar *)"rel",
+			"alternate", (xmlChar *)"href")))       //ATOM
+				link = g_strdup(_("No Information"));
 	}
 
 	id = (gchar *)layer_find (top, (gchar *)"id",				//ATOM
@@ -1071,7 +1076,7 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 				g_free(q2);
 			}
 			g_free(qsafe);
-		} else {			//then RSS or RDF
+	} else {			//then RSS or RDF
 			xmlNodePtr source;
 			source = layer_find_pos(top, "source", "author");
 			//try the source construct
@@ -1082,12 +1087,6 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 				q = g_strdup(layer_find (top, "author",
 					layer_find (top, "creator", NULL)));	//this catches dc:creator too. wrong!
 			}
-			if (q) {
-				GString *s = rss_strip_html(q);
-				q = s->str;
-				g_string_free(s, 0);
-			}
-
 			//we might end with a subject containing nothing but spaces
 			if (q) g_strstrip (q);
 
@@ -1111,7 +1110,7 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 				if (q2) g_free(q2);
 				if (q3) g_free(q3);
 			}
-		}
+	}
 		//FIXME this might need xmlFree when namespacing
 		b = (gchar *)layer_find_tag (top, "content",		//we prefer content first		  <--
 				layer_find_tag (top, "description",	//it seems description is rather shorten version of the content, so |
@@ -1143,6 +1142,7 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 		//handle multiple enclosures
 		encl = (gchar *)layer_find_innerelement(top, "enclosure", "url",	// RSS 2.0 Enclosure
 			layer_find_innerelement(top, "link", "enclosure", NULL));		// ATOM Enclosure
+
 		//handle screwed feeds that set url to "" (feed does not validate!)
 		if (encl && !strlen(encl)) {
 			g_free(encl);
@@ -1150,7 +1150,10 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 		}
 		//handle attatchments (can be multiple)
 		attachments = layer_find_tag_prop(top, "media", "url");
-
+		if (!attachments)
+			attachments = layer_query_find_all_prop (top,
+				"link", (xmlChar *)"rel",
+				"enclosure", (xmlChar *)"href");
 
 //                char *comments = g_strdup(layer_find (top, "comments", NULL));	//RSS,
 		comments = (gchar *)layer_find_ns_tag(top, "wfw", "commentRss", NULL); //add slash:comments
@@ -1212,47 +1215,55 @@ void
 refresh_mail_folder(CamelFolder *mail_folder)
 {
 	mail_refresh_folder(mail_folder, NULL, NULL);
+#if (DATASERVER_VERSION >= 2033001)
+		camel_folder_synchronize (mail_folder, FALSE, G_PRIORITY_DEFAULT,
+			NULL, NULL, NULL);
+#else
 	camel_folder_sync(mail_folder, FALSE, NULL);
+#endif
 	camel_folder_thaw(mail_folder);
 }
 
-gchar *
-update_channel(RDF *r)
+GQueue *
+display_channel_items_sync(AsyncData *asyncr)
 {
 	FILE *fr, *fw;
-	guint i;
-	gchar *sender;
-	xmlNodePtr el;
-	gchar *subj;
-	create_feed *CF;
+	RDF *r = asyncr->r;
+	GQueue *status_msg = asyncr->status_msg;
+	GtkWidget *progress = r->progress;
 	gchar *chn_name = r->title;
 	gchar *url = r->uri;
 	GArray *item = r->item;
-	GtkWidget *progress = r->progress;
-	gchar *buf, *safes, *feed_dir, *feed_name;
-	gchar *msg;
-	gboolean freeze = FALSE;
 	CamelFolder *mail_folder = NULL;
+	gchar *sender;
+	gboolean freeze = FALSE;
 	gchar *article_uid = NULL;
+	gchar *safes, *feed_dir, *feed_name;
+	gchar *msg, *safchn;
+	create_feed *CF;
+	gchar *subj;
+	xmlNodePtr el;
+	guint i;
 
 	safes = encode_rfc2047(chn_name);
-	sender = g_strdup_printf("%s <%s>", safes, chn_name);
+	safchn = g_strchomp (g_strdup(chn_name));
+	sender = g_strdup_printf("%s <%s>", safes, safchn);
+	g_free(safchn);
 	g_free(safes);
 
 	migrate_crc_md5(chn_name, url);
-	buf = gen_md5(url);
+	r->feedid = gen_md5(url);
 	feed_dir = rss_component_peek_base_directory();
 	if (!g_file_test(feed_dir, G_FILE_TEST_EXISTS))
 		g_mkdir_with_parents (feed_dir, 0755);
 
-	feed_name = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", feed_dir, buf);
+	feed_name = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", feed_dir, r->feedid);
 	g_free(feed_dir);
 
 	fr = fopen(feed_name, "r");
 	fw = fopen(feed_name, "a+");
 
 	for (i=0; NULL != (el = g_array_index(item, xmlNodePtr, i)); i++) {
-		update_sr_message();
 		update_progress_text(chn_name);
 		if (rf->cancel || rf->cancel_all || rf->display_cancel)
 			break;
@@ -1276,7 +1287,7 @@ update_channel(RDF *r)
 			r, &article_uid);
 		g_array_append_val(r->uids, article_uid);
 		if (!CF) continue;
-		CF->feedid = g_strdup(buf);
+		CF->feedid = g_strdup(r->feedid);
 		CF->sender = g_strdup(sender);
 		if (r->prefix)
 			CF->full_path = g_build_path(
@@ -1288,34 +1299,36 @@ update_channel(RDF *r)
 		if (!mail_folder) {
 			mail_folder = check_feed_folder(CF->full_path);
 		}
-
 		subj = g_strdup(CF->subj);
 
-		while (gtk_events_pending())
-			gtk_main_iteration ();
-
 		ftotal++;
-		if (CF->encl) {
-			process_enclosure(CF);
-		} else if (g_list_length(CF->attachments)) {
-			process_attachments(CF);
-		} else {
-			if (!freeze) {
-				camel_folder_freeze(mail_folder);
-				freeze = TRUE;
+		if (gconf_client_get_bool(rss_gconf, GCONF_KEY_DOWNLOAD_ENCLOSURES, NULL)) {
+			if (CF->encl) {
+				process_enclosure(CF);
+				goto done;
+			} else if (g_list_length(CF->attachments)) {
+				process_attachments(CF);
+				goto done;
 			}
-			create_mail(CF);
-			write_feed_status_line(
-				CF->feed_fname, CF->feed_uri);
-			free_cf(CF);
 		}
-		farticle++;
+
+		if (!freeze) {
+			camel_folder_freeze(mail_folder);
+			freeze = TRUE;
+		}
+		create_mail(CF);
+		write_feed_status_line(
+			CF->feed_fname, CF->feed_uri);
+		free_cf(CF);
+
+done:		farticle++;
 		d("put success()\n");
-		update_status_icon(chn_name, subj);
+		update_status_icon_text(status_msg, chn_name, subj);
 		g_free(subj);
 	}
 	if (freeze)
 		refresh_mail_folder(mail_folder);
+
 	if (mail_folder) {
 		if ((rf->import || feed_new)
 		&& (!rf->cancel && !rf->cancel_all && !rf->display_cancel)) {
@@ -1335,7 +1348,71 @@ update_channel(RDF *r)
 	if (fw) fclose(fw);
 
 	g_free(feed_name);
-	return buf;
+	return status_msg;
+}
+
+static void
+display_channel_items_thread (GSimpleAsyncResult *simple,
+	GObject *object,
+	GCancellable *cancellable)
+{
+	AsyncData *asyncr;
+	GError *error = NULL;
+
+	asyncr = g_simple_async_result_get_op_res_gpointer (simple);
+	asyncr->status_msg = display_channel_items_sync(asyncr);
+
+	if (error != NULL) {
+		g_simple_async_result_set_from_error (simple, error);
+		g_error_free (error);
+	}
+}
+
+void
+asyncr_context_free(AsyncData *asyncr)
+{
+	d("free r-> components\n");
+	if (asyncr->r->maindate)
+		g_free(asyncr->r->maindate);
+	g_array_free(asyncr->r->item, TRUE);
+	g_free(asyncr->r->feedid);
+	if (asyncr->r->uids)
+		g_array_free(asyncr->r->uids, TRUE);
+	if (asyncr->r->cache)
+		xmlFreeDoc(asyncr->r->cache);
+	if (asyncr->r->type)
+		g_free(asyncr->r->type);
+	if (asyncr->r->version)
+		g_free(asyncr->r->version);
+	g_free(asyncr->r);
+	g_free(asyncr);
+}
+
+
+void
+display_channel_items (RDF *r,
+	gint io_priority,
+	GCancellable *cancellable,
+	GAsyncReadyCallback callback,
+	gpointer user_data)
+{
+	GSimpleAsyncResult *simple;
+	AsyncData *data;
+
+	data = g_new0(AsyncData, 1);
+	data->r = r;
+	data->status_msg = user_data;
+
+	simple = g_simple_async_result_new (
+		NULL, callback, user_data, display_channel_items);
+
+	g_simple_async_result_set_op_res_gpointer (
+		simple, data, (GDestroyNotify)asyncr_context_free);
+
+	g_simple_async_result_run_in_thread (
+		simple, display_channel_items_thread, io_priority, cancellable);
+
+	g_object_unref (simple);
 }
 
 gchar *
