@@ -21,9 +21,13 @@
 
 #include <mail/em-format-html.h>
 
-gchar *update_channel(RDF *r);
+void display_channel_items (RDF *r,
+	gint io_priority,
+	GCancellable *cancellable,
+	GAsyncReadyCallback callback,
+	gpointer user_data);
+
 xmlDoc *rss_html_url_decode(const char *html, int len);
-GString *rss_strip_html (gchar *string);
 
 const char *layer_find (xmlNodePtr node,
 			const char *match,
@@ -50,6 +54,13 @@ layer_query_find_prop (xmlNodePtr node,
 		const char *attrprop,
 		xmlChar *prop);
 
+GList *
+layer_query_find_all_prop (xmlNodePtr node,
+		const char *match,
+		xmlChar *attr,
+		const char *attrprop,
+		xmlChar *prop);
+
 const char *layer_find_tag (xmlNodePtr node,
 			const char *match,
 			const char *fail);
