@@ -17,7 +17,9 @@
  */
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <camel/camel.h>
+#include <libedataserver/e-data-server-util.h>
 #include <mail/em-folder-tree.h>
 #include <mail/em-format-html.h>
 #include <sys/time.h>
@@ -41,6 +43,8 @@ gchar *pixfile;
 char *pixfilebuf;
 gsize pixfilelen;
 extern GHashTable *icons;
+GHashTable *missing;
+
 void
 #if LIBSOUP_VERSION < 2003000
 finish_image_feedback (SoupMessage *msg, FEED_IMAGE *user_data);
@@ -49,7 +53,7 @@ finish_image_feedback (SoupSession *soup_sess, SoupMessage *msg, FEED_IMAGE *use
 #endif
 
 void
-rss_load_images(void)
+rss_init_images(void)
 {
 	/* load transparency */
 	pixfile = g_build_filename (EVOLUTION_ICONDIR,
@@ -63,6 +67,13 @@ rss_load_images(void)
 			NULL);
 }
 
+void
+rss_finish_images(void)
+{
+	if (missing)
+		g_hash_table_destroy(missing);
+}
+
 void
 update_feed_image(RDF *r)
 {
@@ -84,11 +95,7 @@ update_feed_image(RDF *r)
 	if (!g_file_test(feed_file, G_FILE_TEST_EXISTS)) {
 	if (image) {		//we need to validate image here with load_pixbuf
 		CamelStream *feed_fs = camel_stream_fs_new_with_name(feed_file,
-#if EVOLUTION_VERSION < 23191
-			O_RDWR|O_CREAT, 0666);
-#else
 			O_RDWR|O_CREAT, 0666, NULL);
-#endif
 		dup_auth_data(r->uri, image);
 		fi->feed_fs = feed_fs;
 		fi->key = g_strdup(key);
@@ -280,8 +287,32 @@ finish_image_feedback (SoupSession *soup_sess, SoupMessage *msg, FEED_IMAGE *use
 #endif
 {
 	CamelStream *stream = NULL;
+	gchar *mime_type;
 	stream = rss_cache_add(user_data->url);
 	finish_image(soup_sess, msg, stream);
+	if (!missing)
+		missing = g_hash_table_new_full(
+			g_str_hash, g_str_equal, g_free, NULL);
+
+	mime_type = g_content_type_guess(NULL,
+			(guchar *)msg->response_body->data,
+			msg->response_body->length, NULL);
+
+	if (503 == msg->status_code || //handle this timedly fasion
+	    404 == msg->status_code || //NOT FOUND
+	    400 == msg->status_code || //bad request
+	      2 == msg->status_code || //STATUS_CANT_RESOLVE
+	      1 == msg->status_code || //TIMEOUT (CANCELLED) ?
+	      7 == msg->status_code ||// STATUS_IO_ERROR
+		msg->response_body->length || //ZERO SIZE
+	/*FIXME mime type here could be wrong */
+	0 != g_ascii_strncasecmp (mime_type, "image/", 6)) { //ZERO SIZE
+			g_hash_table_insert(missing,
+				g_strdup(user_data->url),
+				GINT_TO_POINTER(1));
+		}
+	g_free(mime_type);
+
 	if (user_data->data == current_pobject)
 #if EVOLUTION_VERSION >= 23190
 		em_format_queue_redraw((EMFormat *)user_data->data);
@@ -319,12 +350,16 @@ finish_image (SoupSession *soup_sess, SoupMessage *msg, CamelStream *user_data)
 				msg->response.length);
 #else
 		if (msg->response_body->data) {
-			camel_stream_write(user_data,
+#if (DATASERVER_VERSION >= 2033001)
+			camel_stream_write (user_data,
 				msg->response_body->data,
-#if EVOLUTION_VERSION < 23191
-				msg->response_body->length);
-			camel_stream_close(user_data);
+				msg->response_body->length,
+				NULL,
+				NULL);
+			camel_stream_close(user_data, NULL, NULL);
 #else
+			camel_stream_write (user_data,
+				msg->response_body->data,
 				msg->response_body->length,
 				NULL);
 			camel_stream_close(user_data, NULL);
@@ -337,9 +372,9 @@ finish_image (SoupSession *soup_sess, SoupMessage *msg, CamelStream *user_data)
 #endif
 		}
 	} else {
-#if EVOLUTION_VERSION < 23191
-		camel_stream_write(user_data, pixfilebuf, pixfilelen);
-		camel_stream_close(user_data);
+#if (DATASERVER_VERSION >= 2033001)
+		camel_stream_write (user_data, pixfilebuf, pixfilelen, NULL, NULL);
+		camel_stream_close (user_data, NULL, NULL);
 #else
 		camel_stream_write(user_data, pixfilebuf, pixfilelen, NULL);
 		camel_stream_close(user_data, NULL);
@@ -364,15 +399,9 @@ finish_create_icon (SoupSession *soup_sess, SoupMessage *msg, FEED_IMAGE *user_d
 	if (404 != msg->status_code) {
 		CamelStream *feed_fs = camel_stream_fs_new_with_name(
 					user_data->img_file,
-#if EVOLUTION_VERSION < 23191
-					O_RDWR|O_CREAT, 0666);
-#else
 					O_RDWR|O_CREAT, 0666, NULL);
-#endif
 		finish_image(soup_sess, msg, feed_fs);
-#if (EVOLUTION_VERSION >= 22703)
 		display_folder_icon(evolution_store, user_data->key);
-#endif
 	}
 	g_free(user_data->key);
 	g_free(user_data);
@@ -388,14 +417,11 @@ finish_create_icon_stream (
 #endif
 {
 	finish_image(soup_sess, msg, user_data->feed_fs);
-#if (EVOLUTION_VERSION >= 22703)
 	display_folder_icon(evolution_store, user_data->key);
-#endif
 	g_free(user_data->key);
 	g_free(user_data);
 }
 
-#if (EVOLUTION_VERSION >= 22703)
 gboolean
 display_folder_icon(GtkTreeStore *tree_store, gchar *key)
 {
@@ -419,14 +445,23 @@ display_folder_icon(GtkTreeStore *tree_store, gchar *key)
 
 	if (pixbuf) {
 		gchar *name = g_hash_table_lookup(rf->hrname_r, key);
+		gchar *folder_name = lookup_feed_folder(name);
 		gchar *full_name = g_build_path(
 					G_DIR_SEPARATOR_S,
 					get_main_folder(),
-					lookup_feed_folder(name),
+					folder_name,
 					NULL);
+		g_free(folder_name);
+/*folder isn't created yet?*/
+#if (DATASERVER_VERSION >= 2033001)
+		rssi = camel_store_get_folder_info_sync (
+					store,
+					full_name, 0, NULL, NULL);
+#else
 		rssi = camel_store_get_folder_info (
 					store,
 					full_name, 0, NULL);
+#endif
 		if (!rssi) {
 			g_free(full_name);
 			result = FALSE;
@@ -472,7 +507,6 @@ out:	g_free(img_file);
 	g_free(feed_dir);
 	return result;
 }
-#endif
 
 gchar *
 decode_image_cache_filename(gchar *name)
@@ -492,15 +526,17 @@ decode_image_cache_filename(gchar *name)
 gboolean image_is_valid(gchar *image);
 
 gboolean
-file_is_image(gchar *image);
+file_is_image(gchar *image, gboolean cleanup);
 
 gboolean
-file_is_image(gchar *image)
+file_is_image(gchar *image, gboolean cleanup)
 {
 	gchar *mime_type, *contents;
 	gsize length;
 	gboolean result = TRUE;
 
+	g_return_val_if_fail(image != NULL, FALSE);
+
 	if (!g_file_test(image, G_FILE_TEST_EXISTS))
 		return FALSE;
 
@@ -515,8 +551,15 @@ file_is_image(gchar *image)
 	mime_type = g_content_type_guess(NULL,
 			(guchar *)contents, length, NULL);
 	/*FIXME mime type here could be wrong */
-	if (g_ascii_strncasecmp (mime_type, "image/", 6))
+	if (g_ascii_strncasecmp (mime_type, "image/", 6)) {
+		struct stat st;
+		if (cleanup) {
+			g_stat(image, &st);
+			if (st.st_size == 0)
+				g_unlink(image);
+		}
 		result = FALSE;
+	}
 	g_free(mime_type);
 	g_free(contents);
 	return result;
@@ -531,12 +574,12 @@ gchar *
 verify_image(gchar *uri, EMFormatHTML *format)
 {
 	gchar *nurl, *turl;
-	gchar *base_dir, *feed_dir, *name;
+	gchar *feed_dir, *name;
 	gchar *scheme, *tname;
 	gchar *result = NULL;
 	gchar *duri = NULL;
 
-	g_return_val_if_fail(uri != NULL, NULL);
+	if (!uri) return NULL;
 
 	if (strstr(uri, "img:"))
 		duri = decode_image_cache_filename(uri);
@@ -545,15 +588,28 @@ verify_image(gchar *uri, EMFormatHTML *format)
 			duri = g_strdup(uri);
 	}
 
-	if (!g_file_test(duri, G_FILE_TEST_EXISTS)) {
+	if (!missing)
+		missing = g_hash_table_new_full(
+			g_str_hash, g_str_equal, g_free, NULL);
+
+	if (g_file_test(duri, G_FILE_TEST_EXISTS)) {
+		struct stat st;
+		g_stat(duri, &st);
+		if (st.st_size == 83) {
+			if (g_hash_table_lookup(missing, g_path_get_basename (duri)))
+				goto out;
+			g_unlink(duri);
+			d("retrying file:%s\n", duri);
+		}
+	}
+
+out:	if (!g_file_test(duri, G_FILE_TEST_EXISTS)) {
 			camel_url_decode((gchar *)uri);
 			//FIXME lame method of extracting data cache path
 			//there must be a function in camel for getting data cache path
-			base_dir = rss_component_peek_base_directory();
 			feed_dir = g_build_path(G_DIR_SEPARATOR_S,
-				base_dir,
-				"static",
-				"http",
+				e_get_user_cache_dir(),
+				"rss",
 				NULL);
 			scheme = g_uri_parse_scheme(uri);
 			/* calling fetch_image_redraw with link NULL
@@ -573,7 +629,6 @@ verify_image(gchar *uri, EMFormatHTML *format)
 				name = fetch_image_redraw(uri, NULL, format);
 				g_free(scheme);
 			}
-			g_free(base_dir);
 			if (name) {
 				tname = decode_image_cache_filename(name);
 				g_free(name);
@@ -582,7 +637,7 @@ verify_image(gchar *uri, EMFormatHTML *format)
 #else
 				result = g_strdup(tname);
 #endif
-				if (!file_is_image(tname)) {
+				if (!file_is_image(tname, 1)) {
 					g_free(tname);
 					goto fail;
 				}
@@ -593,7 +648,7 @@ verify_image(gchar *uri, EMFormatHTML *format)
 				g_free(duri);
 			return result;
 	} else {
-		if (!file_is_image(duri))
+		if (!file_is_image(duri, 1))
 			goto fail;
 /*
  * appears the default has changed in efh_url_requested
@@ -667,6 +722,7 @@ fetch_image_redraw(gchar *url, gchar *link, gpointer data)
 	cache_file = rss_cache_get_filename(intern);
 	d("fetch_image_redraw() tmpurl:%s, intern: %s\n",
 		tmpurl, cache_file);
+
 	if (!g_file_test (cache_file, G_FILE_TEST_EXISTS)) {
 		d("image cache MISS\n");
 		if (data) {
